# $Id: drupalorg.rb,v 1.1 2008/05/04 06:42:47 mooffie Exp $

# @file
# Handler that fixes drupal.org's inaccessible pages.

require 'handlers/sax'

module Proxy
module Handlers

# This handler fixes two problems with drupal.org's HTML:
# - Huge selectboxes, having ID #edit-project-info-pid, are trimmed
#   to show the selected option only. Ditto for ID #edit-projects, which
#   is the selectbox shown on a project's issue listing.
# - All radios are removed from the projects subsription form (see
#   http://drupal.org/project/issues/subscribe-mail).
class Drupalorg < Proxy::Handlers::SAX

  def initialize
    # initialize some state variables
    @in_huge_select = false
    @in_selected_option = false
    @in_subscribe_form = false
  end

  def self.wants_to_handle? request_headers
    # we wish to handle only drupal.org's pages
    request_headers.any? {|h| h[:name] == 'host' and ['drupal.org', 'www.drupal.org'].include? h[:value]}
  end

  #
  # SAX handlers
  #
  
  include Proxy::Handlers::SAXListener
 
  def tag_start tag, attrs

    @in_selected_option = true if tag == 'option' and attrs['selected']

    if @in_subscribe_form
      # what hangs the browser is the many radio buttons
      if tag == 'input'
        @output << (attrs['checked'] ? '[*]' : '[&nbsp;]')
      else
        super
      end
    elsif @in_huge_select and not @in_selected_option
      # output nothing; don't show all thousands of <options> here...
    else
      super
    end

    @in_subscribe_form = true if tag == 'form' and attrs['action'] =~ %r{/project/issues/subscribe-mail$}

    @in_huge_select = true if tag == 'select' and ['edit-project-info-pid','edit-projects'].include? attrs['id']
  end

  def tag_end tag

    @in_huge_select = false if tag == 'select'

    if @in_huge_select and not @in_selected_option
      # output nothing
    else
      super
    end

    @in_selected_option = false if tag == 'option'

    @in_subscribe_form = false if tag == 'table'
  end

  def text s
    if @in_huge_select and not @in_selected_option
      # output nothing
    else
      super
    end
  end    
end # class Drupalorg

end # module Handlers
end # module Proxy
