# $Id: sax.rb,v 1.1 2008/05/04 06:42:47 mooffie Exp $

# @file
# Handler for HTML documents, using a SAX-like API. In itself does nothing.
# Intended for subclassing.

require 'handlers/text'

# We're currently using REXML. It has two drawbacks:
# (1) slow;
# (2) doesn't handle malformed HTML.
# So we should implement a SAX parser ourselves.
require 'rexml/document'
require 'rexml/streamlistener'

module Proxy
module Handlers

class SAX < Proxy::Handlers::Text
  
  # Whether this handler would like to handle the body
  def wants_to_handle_body? response_headers
    return @content_type && @content_type['text/html']
  end

  def handle_response_headers response_headers
    if @content_type && @content_type['text/html']
      # Report 'Content-Length' back if and only if it's not a
      # text/html file, because our filtering may change its size.
      response_headers.replace response_headers.reject { |header| header[':name'] == 'content-length' }
    end
  end

  def handle_body input, output
    @output = output
    REXML::Document.parse_stream input, self
  end
end # class SAX

#
# This is a basic SAX listener that simply writes its input to the output.
# Derived classes should modify the document as they wish.
#
module SAXListener 

  include REXML::StreamListener 

  def tag_start tag, attrs
    @output << '<' << tag
    attrs.each { |n,v|
      @output << ' ' << REXML::Attribute.new(n, v).to_string
    }
    # browsers have probelms with <br></br>, so we're doing <br />
    @output << ' /' if tag == 'br'
    @output << '>'
  end
  
  def doctype *all
    @output << "<!DOCTYPE #{all.join(' ')}>"
  end

  def comment s
    @output << "<!--#{s}-->"
  end

  def cdata s
    @output << "<![CDATA[#{s}]]>"
  end

  def text s
   s = s.gsub('>', '&gt;').gsub('<', '&lt;');
   @output << s
  end
  
  def tag_end tag
    return if tag == 'br' # see comment at tag_start()
    @output << "</#{tag}>"
  end

end # module SAXListener

end # module Handlers
end # module Proxy
