# $Id: text.rb,v 1.1 2008/05/04 06:42:47 mooffie Exp $

# @file
# Handler for plain content. Intended for subclassing.

module Proxy
module Handlers

#
# The Text handler makes it possible for derived classed to "see" the
# docuemnt. It does this by turning off compression, chunking, and making
# the request a dumb HTTP/1.0 one.
#
class Text < Proxy::Handler

  EXCLUDED_HEADERS = [
    # Remove the following headers from the request
    'accept-encoding',  # so we won't receive "Content-encoding: gzip", which we can't parse
    'keep-alive',       # various "keep-alive" headers
    'connection',
    'proxy-connection',
  ]

  def handle_request_headers request_headers
    request_headers.replace request_headers.reject { |header|
      EXCLUDED_HEADERS.include? header[:name]
    }
    # We don't support HTML/1.1 (most importantly, we don't support
    # "Transfer-Encoding: chunked"), so we rewrite the request as HTTP/1.0
    request_headers[0][:source] = request_headers[0][:source].sub(/\sHTTP\/\d+\.\d+/, ' HTTP/1.0')
  end
end

end # module Handlers
end # module Proxy
